(function($) {

    var ControlRepeatSelectView = elementor.modules.controls.BaseData.extend({
        onReady: function() {
    
            var self = this;
            var select = self.$el.find('select');
            
            select.selectize({
                items: self.getControlValue(),
                plugins: ['remove_button'],
                mode: 'multi',
                hideSelected: false,
                duplicates: true,
                inputClass: 'selectize-input elementor-input-style',
                dropdownClass: 'selectize-dropdown select2-dropdown'
            });
    
        }
    
    } );
    
    elementor.addControlView('repeatselect', ControlRepeatSelectView);
    
    var ControlHtml5SortableView = elementor.modules.controls.BaseData.extend({
        onReady: function() {
    
            var self = this;
            var list = self.$el.find('.elementor-control-html5sortable');
    
            self.setValues(list);
            
            sortable(list)[0].addEventListener('sortupdate', function(e) {
                var value = self.getValues(e.detail.destination.items);
                self.setValue(value);
            });
    
        },
        getValues: function(items) {
            var arr = [];
            for (var i = 0; i < items.length; i++) {
                arr.push($(items[i]).attr('data-value'));
            }
            return arr;
        },
        setValues: function(list) {
    
            var values = this.getControlValue();
            var items = [];
    
            for (var i = 0; i < values.length; i++) {
                items.push($(list).find('[data-value=' + values[i] + ']')[0]);
            }
    
            list.empty().append(items);
        }
    
    } );
    
    elementor.addControlView('html5sortable', ControlHtml5SortableView);
    
    elementor.channels.editor.on('emage:editor:activate emage:editor:deactivate', function(event, model) {
    
        var button = event.$el.find('.elementor-button');
        var codeInput = event.$el.closest('#elementor-controls').find('input[data-setting="license_purchase_code"]');
        var licenseInput = event.$el.closest('#elementor-controls').find('input[data-setting="license"]');
        var noticeDiv = event.$el.closest('#elementor-controls').find('.elementor-control-license_notice').find(".elementor-control-raw-html");
    
        var eventdata = button.attr('data-event');
        var action = (eventdata.indexOf('deactivate') !== -1) ? 'deactivate' : 'activate';
    
        var data = {
            action: 'emage_license',
            license_action: action,
            code: codeInput.val()
        };
    
        button.addClass('elementor-button-state');
        button.find('.publish-label').hide();
    
        jQuery.post(emage.ajax_url, data,
            function(response) {
                button.removeClass('elementor-button-state');
                button.find('.publish-label').show();
                noticeDiv.html('<div style="color:#0b5885;background-color:#d0eeff;border-color:#bee7ff;padding: .75rem 1.25rem;border-radius: .25rem;line-height: 1.3em;border: 1px solid #bbcff5;">' + response.message + '</div>');
                licenseInput.val(response.license).trigger('input');
                if (response.license !== 'false') {
                    var obLicense = data.code.substr(0,8) + '-xxxx-xxxx-xxxx-xxxxxxxxxxxx';
                    codeInput.val(obLicense).trigger('input');
                } else {
                    codeInput.val('').trigger('input');
                }
            }
        );
        
    });

    elementor.hooks.addAction( 'panel/open_editor/widget/emage_post_grid', ( panel, model ) => {
        const settingsModel = model.get( 'settings' );
        var type = '';
        type = $('[data-setting="post_type"]').val();

        // $('[data-setting="categories"]').val();

        // on editor open trigger ajax
        updateTerms(type, settingsModel);

        // trigger ajax on change of post type
        settingsModel.on( 'change', ( changedModel ) => {
            // need to check all 3 occurance to avoid recursive call because of change event trigger inside the change event
            if(changedModel.changed['post_type'] != undefined && !changedModel.changed['categories'] && !changedModel.changed['tags']){

                type = changedModel.changed['post_type'];
                // Need to empty the cat and atg for previous post types
                // triggering change inside a change event to set result for current post types
                $('[data-setting="categories"]').empty().trigger('change');
                $('[data-setting="tags"]').empty().trigger('change');

                updateTerms(type, settingsModel);
            }
        });
    });

    function updateTerms(type, settingsModel){
        // disable categories and tags dropdown and add loader to denote update from ajax
        $('.elementor-control-categories, .elementor-control-tags').find('select').attr('disabled', true);
        $('.elementor-control-categories .elementor-control-title, .elementor-control-tags .elementor-control-title').after('<span class="elementor-control-spinner"><i class="eicon-spinner eicon-animation-spin"></i>&nbsp;</span>');

        var data = {
            action: 'emage_post_terms',
            post_type: type
        };

        $.post(emage.ajax_url, data, function(response) {
            var categories = Object.entries(response['cat']);
            var tags = Object.entries(response['tag']);

            var datasets = [categories, tags];
            datasets.forEach(function(item, index){
                var set =  index == 0 ? settingsModel.attributes.categories : settingsModel.attributes.tags;
                var dropdown = index == 0 ? $('[data-setting="categories"]') : $('[data-setting="tags"]');

                for (const [slug, name] of item) {
                    var selected = (settingsModel.attributes.categories).includes(slug) ? 'selected' :'';
                    dropdown.append('<option '+selected+' value="'+slug+'">'+name+'</option>');
                }
                if(!set.length > 0 && dropdown){ 
                    dropdown[0].selectedIndex = -1;
                }
            });

            // remove loader and disabled attribute
            $('.elementor-control-categories, .elementor-control-tags').find('.elementor-control-spinner').remove();
            $('.elementor-control-categories, .elementor-control-tags').find('select').removeAttr('disabled');

        });
    }

})(jQuery);